c=========================================================================	
c	calculating QCD Wilson coefficients for effective theory
c	according to eq. 5.130 of Ph.D. thesis 
c	"Phenomenology of composite Higgs models" by Elisabetta Furlan
c -------       
	subroutine calculate_qcd_wilson_coefficients(mur)
	implicit none
	!
	double precision double_mq(100),Yq(100)
	double complex Wq(100),Xq(100),Mq_square_complex(100)
	common /XWq/ Xq,Wq,Mq_square_complex,double_mq,Yq,
     # number_of_quarks   
      !
	
	integer number_of_quarks,i,j
	double precision Y_0,Y_1,Y_2,nh,Lm,Lm2
	double precision mur,heavy_quark_threshold
	double precision  NF
	double precision yi,yj,mi,mj,old_AC2
	
      double complex polylog_piece,DMR,HPL2,HPL3,miOmj,mjOmi,cmi,cmj
	double complex AC0,AC1,AC2
	common /commonWCQCD/ AC0,AC1,AC2
	!
	NF=5d0;
	Y_0=0d0
	Y_1= 0d0
	Y_2=0d0
	nh=0d0
	Lm=0d0
	Lm2=0d0
	polylog_piece=dcmplx(0d0,0d0)
	heavy_quark_threshold=50d0
	do i=1,number_of_quarks
		if (double_mq(i).gt.heavy_quark_threshold) then 
			Y_0=Y_0+Yq(i)
			Y_1 = Y_1 + Yq(i)*log(double_mq(i)/mur)
			Y_2 = Y_2 + Yq(i)*log(double_mq(i)/mur)**2
			nh=nh+1d0
			Lm=Lm+log(double_mq(i)/mur)
			Lm2=Lm2+log(double_mq(i)/mur)**2
			
                  do j=i+1,number_of_quarks
				if (double_mq(j).gt.heavy_quark_threshold) then 
                  if (dabs(double_mq(i)-double_mq(j)).gt.1d-14) then
                                    
				yi=Yq(i)
				yj=Yq(j)
				mi=double_mq(i)
				mj=double_mq(j)
                        cmi=dcmplx(mi,1d-16)
                        cmj=dcmplx(mj,1d-16)
                        
                        
                        DMR=cmi-cmj
                        miOmj=cmi/cmj
                        mjOmi=cmj/cmi
	polylog_piece = polylog_piece  
     #			+(yi-yj)* (57d0/128d0*(mi**2/mj**2-mj**2/mi**2)
     #					+(57d0/128d0*mi**2/mj**2
     #						+57d0/128d0*mj**2/mi**2
     #						+43d0/32d0
     #					)*log(mi/mj)
     #					+57d0/256d0*(mi**6+mj**6)/mi**2
     #					/mj**2/(mi**2-mj**2)*log(mi/mj)**2
     #				)
     #			-log(mi/mj)**2*(73d0/256d0*(yi+yj)
     #					+23d0/128d0*(yi*mi**2-yj*mj**2)/(mi**2-mj**2)
     #					+3d0/512d0*(mi**2-mj**2)*
     #						(19d0*mi**4
     #						+24d0*mi**2*mj**2
     #						+19d0*mj**4)/mi**3/mj**3*
     #							(yj*cdlog(-DMR/(mi+mj))
     #							-yi*cdlog(DMR/(mi+mj))
     #							)
     #					)
     #			-3d0/1024d0*(19d0*mi**6+5d0*mi**4*mj**2
     #					-5d0*mi**2*mj**4-19d0*mj**6
     #					)/mi**3/mj**3
     #				*(    8d0*yi*HPL3(0,0,1,mjOmi)
     #                              -8d0*yj*HPL3(0,0,1,miOmj)
     #                              -yi*HPL3(0,0,1,mjOmi**2)
     #                              +yj*HPL3(0,0,1,miOmj**2)
     #                              -2d0*log(miOmj)*
     #                                    (
     #                                    yi*HPL2(0,1,mjOmi**2)
     #                                    +yj*HPL2(0,1,miOmj**2)
     #                                    -4d0*yi*HPL2(0,1,mjOmi)
     #                                    -4d0*yj*HPL2(0,1,miOmj)
     #                                    )
     #				)
                              endif ! unequal masses
                        endif ! j is heavy
			enddo ! for j in quarks
		endif ! i is heavy
	enddo ! i  in quarks
	
	AC0 = dcmplx( Y_0 ,0d0)
	AC1 = dcmplx( 11.0d0/4d0*Y_0 ,0d0)
	AC2 = dcmplx( -NF*(67d0/96d0*Y_0+2d0/3d0*Y_1)
     #	+Y_0* (1877d0/192d0-77d0/576d0*nH+113d0/96d0*Lm+3d0/8d0*Lm2)
     #	-Y_1* (19d0/8d0+113d0/96d0*nh+3d0/4d0*Lm)
     #	+Y_2* 3d0/8d0*nh
     #	,0d0)
     #		+polylog_piece
      old_AC2=1877d0/192d0*Y_0-77d0/576d0*Y_0**2-19d0/8d0*Lm
     #		-NF*(67d0/96d0*Y_0+2d0/3d0*Lm) 
	!print*," AC2=",AC2
      !print*,"old =",old_AC2
      !call betta_Wilson_coefficients_QCD(mur)
      !print*,"AC0=",AC0," Ac1=",AC1," AC2=",AC2
	!stop
	end
	



c=========================================================================	
c	calculating QCD Wilson coefficients for effective theory
c -------       
	subroutine betta_Wilson_coefficients_QCD(mur)
	implicit none
      !
	double precision double_mq(100),Yq(100)
	double complex Wq(100),Xq(100),Mq_square_complex(100)
	common /XWq/ Xq,Wq,Mq_square_complex,double_mq,Yq,
     # number_of_quarks   
      !
	
	integer number_of_quarks,i,j
	double precision Y_0,Y_1,Y_2,nh,Lm,Lm2
	double precision mur,heavy_quark_threshold
	double precision  NF
	double precision yi,yj,mi,mj
	
      double complex polylog_piece,DMR,HPL2,HPL3,miOmj,mjOmi,cmi,cmj
      !-----------------
	
C	var's I define for the CHM 
	double precision yot0, yot1, yot2, L1, L2
	double precision y1, y2
	double complex  m1R, m2R
	double complex C2diff, C2BSMC
	double precision C2eq
	integer ind1, ind2, nr_heavy
C       NEW
	double complex AC0,AC1,AC2
	common /commonWCQCD/ AC0,AC1,AC2

C       nr_heavy includes the top
C
CNEW	double complex AC0,AC1,AC2
      Nf=5.0
	nH=0d0
C       nH is my yot0!
C SM	Lm = 0d0
C initialise some var's
	L1 = 0d0
	L2 = 0d0
	yot0 = 0d0
	yot1 = 0d0
	yot2 = 0d0
	nr_heavy = 0

	heavy_quark_threshold=50d0
	do i=1,number_of_quarks
        	
		if (double_mq(i).gt.heavy_quark_threshold) then 
			nH=nH+Yq(i)
			nr_heavy = nr_heavy + 1
C SM			Lm = Lm + log(double_mq/mur)
			L1 = L1 + log(double_mq(i)/mur)
			L2 = L2 + ((log(double_mq(i)/mur))**2)
			yot0 = yot0 + Yq(i)/246d0
			yot1 = yot1 + Yq(i)/246d0*log(double_mq(i)/mur)
			yot2 = yot2
     & + Yq(i)/246d0*(log(double_mq(i)/mur))**2
		endif
	enddo

C --- Compute the WC!
      C2diff = dcmplx(0d0, 0d0)
      do ind1 = 1, number_of_quarks - 1
C --- now do the diagrams for different masses
         do ind2 = ind1 + 1, number_of_quarks
	    if ((double_mq(ind1).gt.heavy_quark_threshold).and.
     &(double_mq(ind2).gt.heavy_quark_threshold)) then 
	       if (dabs(double_mq(ind1)-double_mq(ind2)).gt.1d-10) then
                  !print*,"different mass case"
C       y1 = 246d0*quark_list(3,ind1)/quark_list(1,ind1)
C       y2 = 246d0*quark_list(3,ind2)/quark_list(1,ind2)
		  y1 = Yq(ind1)/246.0d0
		  y2 = Yq(ind2)/246.0d0
		  m1R = dcmplx( double_mq(ind1), -1d-8)
		  m2R = dcmplx( double_mq(ind2), -1d-8)
C
C --- I need an im part to pick the correct im part on the HPLs!
C
		  C2diff = C2diff 
     &          + (y1 - y2)*(
     &             (57d0*(m1R**4 - m2R**4))/(128d0*m1R**2*m2R**2) 
     &           + ((172d0 + (57d0*(m1R**4 + m2R**4))/(m1R**2*m2R**2))
     &           *cdlog(m1R/m2R))/128d0 
     &           + (57d0*(m1R**6 + m2R**6)*cdlog(m1R/m2R)**2)
     &             /(256d0*m1R**2*m2R**2*(m1R**2 - m2R**2)))
C --- ends the piece (y1-y2)
     &          - (((73d0*(y1 + y2))/256d0 
     &             + (23d0*(m1R**2*y1 - m2R**2*y2))
     &               /(128d0*(m1R**2 - m2R**2)))*cdlog(m1R/m2R)**2)
C --- ends line 3 page 120 eq 5.130 of (original) thesis
     &          + (3d0/(1024d0*m1R**3*m2R**3))*(m1R**2 - m2R**2)
     &            *(19d0*m1R**4 + 24d0*m1R**2*m2R**2 + 19d0*m2R**4)
     &            *(-2d0*cdlog(m1R/m2R)**2*(
     &                y2*cdlog((m2R - m1R)/(m1R + m2R)) 
     &              - y1*cdlog((m1R - m2R)/(m1R + m2R))) 
     &            + 2d0*cdlog(m1R/m2R)*(
     &                 y2*HPL2(0,1,m1R**2/m2R**2)
     &               + y1*HPL2(0,1,m2R**2/m1R**2)
     &               - 4d0*y2*HPL2(0,1,m1R/m2R) 
     &               - 4d0*y1*HPL2(0,1,m2R/m1R) )
     &            - 8d0*y1*HPL3(0,0,1,m2R/m1R) 
     &            + 8d0*y2*HPL3(0,0,1,m1R/m2R)
     &            + y1*HPL3(0,0,1,m2R**2/m1R**2)
     &            - y2*HPL3(0,0,1,m1R**2/m2R**2) )
		  else
C ---  ends the "different masses check; if masses are =, this 
C ---  C2diff is 0
                  !print*,"equal mass case"
		     C2diff=C2diff+dcmplx(0d0,0d0)
C --- end if checking on masses being equal
		endif
C ---       ends the "if both quarks are heavy" check
	    endif
C --- ends the piece with the "19 m^6 fraction in front 
         !print*,ind1,ind2,C2diff
         enddo
C --- closes the cycle over ind2
	enddo
C --- closes the cycle over ind1

      C2eq = - NF*(67d0/96d0*yot0 + 2d0/3d0*yot1)
     &       + yot0*(1877d0/192d0 - 77d0/576d0*nr_heavy
     &          +113d0/96d0*L1 + 3d0/8d0*L2 )
     &       - yot1*(19d0/8d0 + 113d0/96d0*nr_heavy + 3d0/4d0*L1 )
     &       + 3d0/8d0*nr_heavy*yot2

      C2BSMC = C2eq + C2diff
      
      if (DABS((DIMAG(C2BSMC)/DREAL(C2BSMC))) .GT. 1E-6) then
         print*, '\n\n\n\nWC re,im,ratio', DREAL(C2BSMC),DIMAG(C2BSMC),
     &     DIMAG(C2BSMC)/DREAL(C2BSMC)
C         STOP '\n\n\nThe Wilson coefficient is complex\n\n\n\n\n' 
C         go to 50
      end if

C from old Hgg      C2BSM = DREAL(C2BSMC)
      
	AC2 = C2BSMC*246d0
      !print*,"C2BSMC=",C2BSMC,C2eq,C2diff,'\tAC2=',AC2,C2BSMC
	AC0 = dcmplx(yot0*246d0, 0d0)
	AC1 = dcmplx(yot0*11.0d0/4d0*246d0, 0d0)
      print*,"[in Betta's WC] ",AC0,AC1,AC2
CC "old" 4 gen template - keep it to remember the LO renorm	
CC	AC0 = dcmplx( nH ,0d0)
CC	AC1 = dcmplx( 11.0d0/4d0*nH ,0d0)
CC This is the four-generation like WC Mine is a little more complicated
CC	AC2 = dcmplx( 1877d0/192d0*nH-77d0/576d0*nH**2-19d0/8d0*Lm
CC     #		-NF*(67d0/96d0*nH+2d0/3d0*Lm) ,0d0)
	
	end


	
c=========================================================================	
c	calculating Wilson coefficients for effective theory
c -------       
	subroutine calculate_wilson_coefficients(QQQ,mur)
	implicit none
	double precision WC0LO,WC0NLO,WC0NNLO,WC1NLO,WC1NNLO
	double precision WC2NNLO
	common /WC/  WC0LO,WC0NLO,WC0NNLO,WC1NLO,WC1NNLO,WC2NNLO
	double complex C0,C1,C2,C0_partial,C1_partial,C2_partial
	double complex C2w
	double complex born_sum_heavy,born_pass
	integer Norder
	double precision fitted!,myrealpart
	double complex lambda_QCD,lambda_W
	double precision QQQ,theta0,theta1,theta2
	double precision  L, Lt, Lr, NF
	common /xsection/ L, Lt, Lr, NF
	! getting mtop from main 
	!
	integer number_of_quarks
	double precision double_mq(100),Yq(100)
	double complex Wq(100),Xq(100),Mq_square_complex(100)
	common /XWq/ Xq,Wq,Mq_square_complex,double_mq,Yq,
     # number_of_quarks   
      !
	double precision mtop_wc,mur
c ------- 
	!: effective theory flag to switch between HQET LO/NLO and exact LO/NLO
	integer effective_theory_flag
	common /common_eff_theory_flag/ effective_theory_flag
c -------
	!: K_ewk is a global rescaling factor for the EWK piece. It's set in runcard
	double precision K_ewk
	common /commonKewk/ K_ewk
c -------
	double complex AC0,AC1,AC2
	common /commonWCQCD/ AC0,AC1,AC2
	
	
	mtop_wc=double_mq(1)
	
c	call Wilson_coefficients_QCD(NF,AC0,AC1,AC2,mur)
c	print*,"AC0 ",AC0,1d0
c	print*,"AC1 ",AC1,11d0/4d0
c	print*,"AC2 ",AC2,dcmplx(2777d0/288d0+(19d0/16d0)*Lt
c     #	+NF*((1d0/3d0)*Lt-67d0/96d0)
c     #	,0d0)

	
	
	!note: lambda_QCD is a complex number
	lambda_QCD = born_sum_heavy(QQQ)/AC0 
	!print*,"lambda_QCD ",lambda_QCD
      !stop
	!note: lambda_W is a complex number
	lambda_W = fitted(QQQ,mtop_wc)

	lambda_W= lambda_W*born_pass(QQQ)*K_ewk
	
	C0_partial = lambda_QCD *AC0!* dcmplx(1d0,0d0)
	
	C1_partial = lambda_QCD *AC1!*dcmplx(11d0/4d0,0d0)
	 
	C2_partial = lambda_QCD * AC2
c     #	dcmplx(2777d0/288d0+(19d0/16d0)*Lt
c     #	+NF*((1d0/3d0)*Lt-67d0/96d0)
c     #	,0d0)
	
	C2w = dcmplx(10d0,0d0)
	
	C0 = lambda_QCD * AC0! dcmplx(1d0,0d0) 
     #	+ lambda_W * dcmplx(1d0,0d0) 
	
	C1 = lambda_QCD *AC1!dcmplx(11d0/4d0,0d0) 
     #	+ lambda_W * dcmplx(7d0/6d0,0d0)
	
	C2 = lambda_QCD * AC2
c     #	dcmplx(2777d0/288d0+(19d0/16d0)*Lt
c     #	+NF*((1d0/3d0)*Lt-67d0/96d0)
c     #	,0d0) 
     #	+ lambda_W*C2W
	
	Norder = 1
	
	theta0=1d0
	theta1=1d0
	theta2=1d0
	if (Norder.lt.0) then 
		theta0=0d0
		theta1=0d0
		theta2=0d0
	endif
	
	if (Norder.lt.1) then 
		theta1=0d0
		theta2=0d0
	endif
	
	if (Norder.lt.2) then 
		theta2=0d0
	endif
	if (effective_theory_flag.eq.0) then 
		WC0LO = abs(C0)**2 - theta0 * abs(C0_partial)**2
		WC0NLO = abs(C0)**2 - theta1 * abs(C0_partial)**2
		WC0NNLO = abs(C0)**2 - theta2 * abs(C0_partial)**2
		WC1NLO = 2d0*dreal(C0*C1) 
     #		- theta1 * 2d0*dreal(C0_partial*C1_partial)
		WC1NNLO = 2d0*dreal(C0*C1) 
     #		- theta2 * 2d0*dreal(C0_partial*C1_partial)
		WC2NNLO = abs(C1)**2- theta2*abs(C1_partial)**2 
     #		+ 2d0*dreal(C0*C2) 
     #		- theta2*2d0*dreal(C0_partial*C2_partial)
      elseif (effective_theory_flag.eq.1) then
		WC0LO = abs(C0)**2
		WC0NLO = abs(C0)**2
		WC0NNLO = abs(C0)**2
		WC1NLO = 2d0*dreal(C0*C1) 
		WC1NNLO = 2d0*dreal(C0*C1) 
		WC2NNLO = abs(C1)**2 + 2d0*dreal(C0*C2) 
	endif
      !print*,">>",WC0LO,C0,C0_partial,lambda_QCD,AC0
      !stop
	end
	
c=========================================================================	
c	calculating W(i)=tau(i)= 4 m_i^2 / QQQ^2 where QQQ is the current mass of the higgs
	subroutine calculate_W_X(QQQ)
	implicit none
	double precision QQQ
	!
	double precision double_mq(100),Yq(100)
	double complex Wq(100),Xq(100),Mq_square_complex(100)
	common /XWq/ Xq,Wq,Mq_square_complex,double_mq,Yq,
     # number_of_quarks   
      !

      integer number_of_quarks,i
	      
      do i=1,number_of_quarks
		Wq(i)=4d0*Mq_square_complex(i)/QQQ**2
		Xq(i) = -Wq(i)/(sqrt(1d0-Wq(i))+1d0)**2
      enddo
	end

c=========================================================================	
c	Setting up electroweak masses and couplings 
	subroutine setup_ewk_model(in_number_of_vector_bosons,VBdata)
	implicit none
	! electroweak particle content
	double complex vb_mvsq(100)
	double precision lambda_y(100),vy_up(100),vy_dn(100)
	double precision ay_up(100),ay_dn(100)
	integer vb_type(100)
	integer number_of_VBs
	common /ewk_model_common/ vb_mvsq,lambda_y,vy_up,vy_dn,ay_up,
     $ay_dn,vb_type,number_of_VBs
      double precision VBdata(0:399) !maximum number of VBs: 100
      integer i,iix,in_number_of_vector_bosons
      double precision Gamma_v,delta,cuttoff,double_m
	double precision gw, swsq,cw,mw,mz,vb_Y
	!
	double precision pi,z2,z3
	common /consts/ pi, z2, z3
	!
	!
	double precision Gf
	common /comGf/ Gf
	!
	cuttoff=1e-3
	mw = 80.399d0
	mz = 91.1876d0
	cw= mw/mz
	swsq = (1-cw**2)
	
	gw = dsqrt(4d0*dsqrt(2d0)*Gf)*mw
      number_of_VBs = in_number_of_vector_bosons
	print*,""
      do i=1,number_of_VBs
		double_m = VBdata(4*(i-1)) 
		Gamma_v = VBdata(4*(i-1)+1)
		vb_Y = VBdata(4*(i-1)+2)
		vb_type(i) = VBdata(4*(i-1)+3)
        
		if (Gamma_v/double_m.gt.cuttoff) then 
			vb_mvsq(i) = double_m*dcmplx(double_m,-Gamma_v)
		else
			delta =  1d-16
			vb_mvsq(i) = dcmplx(double_m**2,-delta)            
		endif
		select case (vb_type(i))
		case(1) ! W-like
			lambda_y(i) = 2d0
			vy_up(i) = gw/dsqrt(2d0)*vb_Y
			vy_dn(i) = gw/dsqrt(2d0)*vb_Y
			ay_up(i) = gw/dsqrt(2d0)*vb_Y
			ay_dn(i) = gw/dsqrt(2d0)*vb_Y
		case(2) ! Z-like
			lambda_y(i) = 2d0
			vy_up(i) = gw/cw*(1d0/2d0 - 4d0/3d0*swsq)
			vy_dn(i) = gw/cw*(-1d0/2d0 + 2d0/3d0*swsq)
			ay_up(i) = gw/cw*1d0/2d0
			ay_dn(i) = gw/cw*1d0/2d0
		case default
			print*,"ERROR: unknown type of vector boson :",vb_type(i)
			stop
		end select
c		write(*,"(A,I3,A,3(en16.4,' '),I3)"),"Vector bosons ",i," : ",
c     #	double_m,Gamma_v,vb_Y,vb_type(i)
      enddo
	end


c=========================================================================	

	double precision function myrealpart(z)
	implicit none
	double complex z
	myrealpart = dreal(0.5d0*(z+CONJG(z)))
	!print*,z,myrealpart
	end
	
	


